From 5043fcbc7180d3cdc9efcd9ff957181ee09876b1 Mon Sep 17 00:00:00 2001
From: River Satya <river@squareup.com>
Date: Wed, 22 Jan 2020 09:53:48 -0800
Subject: [PATCH] Fix overflow bug in Base64 decoder

---
 src/main/java/com/nimbusds/jose/util/Base64Codec.java | 11 ++++++++++-
 .../java/com/nimbusds/jose/util/Base64CodecTest.java  |  5 ++++-
 2 files changed, 14 insertions(+), 2 deletions(-)

diff --git a/src/main/java/com/nimbusds/jose/util/Base64Codec.java b/src/main/java/com/nimbusds/jose/util/Base64Codec.java
index 51975594..0782a54d 100644
--- a/src/main/java/com/nimbusds/jose/util/Base64Codec.java
+++ b/src/main/java/com/nimbusds/jose/util/Base64Codec.java
@@ -349,7 +349,7 @@ final class Base64Codec {
 		final int sLen = srcBytes.length;
 
 		// Calculate output length assuming zero bytes are padding or separators
-		final int maxOutputLen = sLen * 6 >> 3;
+		final int maxOutputLen = checkedCast((long)sLen * 6 >> 3);
 
 		// Allocate output array (may be too large)
 		final byte[] dstBytes = new byte[maxOutputLen];
@@ -390,4 +390,13 @@ final class Base64Codec {
 		// Copy dstBytes to new array of proper size
 		return Arrays.copyOf(dstBytes, d);
 	}
+
+	private static int checkedCast(long value) {
+		int result = (int) value;
+		if (result != value) {
+			throw new IllegalArgumentException(value +
+					" cannot be cast to int without changing its value.");
+		}
+		return result;
+	}
 }
diff --git a/src/test/java/com/nimbusds/jose/util/Base64CodecTest.java b/src/test/java/com/nimbusds/jose/util/Base64CodecTest.java
index b8371ae1..cd54abe7 100644
--- a/src/test/java/com/nimbusds/jose/util/Base64CodecTest.java
+++ b/src/test/java/com/nimbusds/jose/util/Base64CodecTest.java
@@ -22,7 +22,7 @@ import java.nio.charset.Charset;
 
 import com.nimbusds.jose.util.Base64Codec;
 import junit.framework.TestCase;
-
+import org.apache.commons.lang.StringUtils;
 
 /**
  * Tests the base 64 codec.
@@ -158,6 +158,9 @@ public class Base64CodecTest extends TestCase {
 		assertEquals("foobar", new String(Base64Codec.decode("Zm9vYmFy"), Charset.forName("utf-8")));
 	}
 
+	public void testDecodeLargeString() {
+		Base64Codec.decode(StringUtils.repeat(" ", Integer.MAX_VALUE / 4));
+	}
 
 	public void testDecodeWithIllegalChars() {
 
-- 
2.25.0

